#include "pgmpeg2.hh"
#include "pgbutton.h"
#include "pgwindow.h"
#include "pgslider.h"
#include "pgcheckbutton.h"

#include "pgnavigator.h"

class PG_MyMPEGPlayerWindow : public PG_MoveableMPEGPLayerWindow {
        bool bhasmoved, binclick, bfs;
        PG_Rect wsize;
public:
        PG_MyMPEGPlayerWindow(char *file, PG_Widget *parent, const PG_Rect & r)
	  : PG_MoveableMPEGPLayerWindow(file, parent, r) {
	  bhasmoved = false;
	  binclick  = false;
	  bfs       = false;
	  wsize = r;
	};

        ~PG_MyMPEGPlayerWindow() { };

protected:
        bool eventMouseButtonDown(const SDL_MouseButtonEvent *button) {
	  if (!binclick)
	    {
	      bhasmoved = false;
	      fprintf(stderr, "%s:%s:%i\n", __FILE__, __FUNCTION__, __LINE__);
	      return PG_MoveableMPEGPLayerWindow::eventMouseButtonDown(button);
	    };
	  if (bhasmoved)
	    {
	      binclick = false;
	      fprintf(stderr, "%s:%s:%i\n", __FILE__, __FUNCTION__, __LINE__);
	      return PG_MoveableMPEGPLayerWindow::eventMouseButtonDown(button);
	    };
	  // PG_MoveableMPEGPLayerWindow::eventMouseButtonUp(button);
	  return true;
	}
  
        bool eventMouseButtonUp(const SDL_MouseButtonEvent *button) {
	  if (bhasmoved)
	    {
	      fprintf(stderr, "%s:%s:%i\n", __FILE__, __FUNCTION__, __LINE__);
	      binclick = false;
	      return PG_MoveableMPEGPLayerWindow::eventMouseButtonUp(button);
	    };
	  if (!binclick)
	    {
	      fprintf(stderr, "%s:%s:%i\n", __FILE__, __FUNCTION__, __LINE__);
	      binclick = true;
	      return PG_MoveableMPEGPLayerWindow::eventMouseButtonUp(button);
	    };
	  PG_Widget* par;
	  par = GetParent();
	  bfs = !bfs;
	  bhasmoved = false;
	  binclick = false;
	  if (!bfs)
	    {
	      fprintf(stderr, "%s:%s:%i %i,%i\n", __FILE__, __FUNCTION__, __LINE__, wsize.w, wsize.h);
	      SizeWidget(wsize.w, wsize.h);
	      fprintf(stderr, "%s:%s:%i %i,%i\n", __FILE__, __FUNCTION__, __LINE__, wsize.x, wsize.y);
	      MoveWidget(wsize.x, wsize.y);
	      fprintf(stderr, "%s:%s:%i\n", __FILE__, __FUNCTION__, __LINE__);
	    }
	  else
	    {
	      fprintf(stderr, "%s:%s:%i\n", __FILE__, __FUNCTION__, __LINE__);
	      wsize.x = this->x;
	      wsize.y = this->y;
	      wsize.h = this->h;
	      wsize.w = this->w;
	      if (par != NULL)
		{
		  par->Hide();
		  fprintf(stderr, "%s:%s:%i\n", __FILE__, __FUNCTION__, __LINE__);
		  MoveWidget(par->x, par->y);
		  fprintf(stderr, "%s:%s:%i\n", __FILE__, __FUNCTION__, __LINE__);
		  SizeWidget(par->w, par->h);
		  fprintf(stderr, "%s:%s:%i\n", __FILE__, __FUNCTION__, __LINE__);
		  // UpdateScreen();
		  // fprintf(stderr, "%s:%s:%i\n", __FILE__, __FUNCTION__, __LINE__);
		  par->Show();
		}
	      else
		{
		  fprintf(stderr, "%s:%s:%i\n", __FILE__, __FUNCTION__, __LINE__);
		  MoveWidget(0, 0);
		  fprintf(stderr, "%s:%s:%i\n", __FILE__, __FUNCTION__, __LINE__);
		  SizeWidget(PG_Application::GetScreenWidth(),
			     PG_Application::GetScreenHeight());
		  fprintf(stderr, "%s:%s:%i\n", __FILE__, __FUNCTION__, __LINE__);
		};
	      fprintf(stderr, "%s:%s:%i\n", __FILE__, __FUNCTION__, __LINE__);
	      return true;
	    };
	  fprintf(stderr, "%s:%s:%i\n", __FILE__, __FUNCTION__, __LINE__);
	  return true;
	}
  
        bool eventMouseMotion (const SDL_MouseMotionEvent *motion) {
	  /* if ((abs(motion->x) > 5) && (abs(motion->y) > 5))
	        { bhasmoved=true; return PG_MoveableMPEGPlayerWindow::eventMouseMotion(motion); };
	  */
	  if (binclick) { fprintf(stderr, "%s:%s:%i\n", __FILE__, __FUNCTION__, __LINE__); };
	  bhasmoved = true;
	  return PG_MoveableMPEGPLayerWindow::eventMouseMotion(motion);
	}
};

// int done = 0;
bool handle_exit(PG_Button* button, PG_Pointer* clientdata)
{
        PG_Application* app = reinterpret_cast<PG_Application*>(clientdata);
        app->Quit();
        // done = 1;
        return true;
}

class PG_Panel : public PG_ThemeWidget
{
public:
	PG_Panel(PG_Widget* parent, const PG_Rect& r) 
	  : PG_ThemeWidget(parent, r) 
	{
		SetBorderSize(2);
	};

};

class PG_PlayerWindow : public PG_Window
{
private:
  bool my_resizeMode;
  Uint8 my_resizeTransparency;
  PG_Point my_resizeDelta;
public:
  PG_MPEGPlayerWindow *player;

  PG_PlayerWindow(PG_Widget* parent, const PG_Rect& r,
		  const char* windowtext,
		  Uint32 flags = WF_DEFAULT,
		  const char* style="Window", int heightTitlebar = 25)
    : PG_Window(parent,r,windowtext,flags,style,heightTitlebar)
  {
    player = new PG_MPEGPlayerWindow(windowtext, this, PG_Rect(2,27,r.w-4,r.h-29));
    player->Show();
    my_resizeMode = false;
  };
protected:
#if 0
  void eventSizeWidget(Uint16 w, Uint16 h)
  {
    PG_Window::eventSizeWidget(w,h);
    player->eventSizeWidget(w-4,h-29);
  }
#endif
  bool eventMouseButtonDown(const SDL_MouseButtonEvent *button)
  {
    PG_Rect ta;
    ta.x = this->x + (this->w - 2);
    ta.y = this->y + (this->h - 2);
    ta.w = 2;
    ta.h = 2;
    
    int x,y;
    x = button->x;
    y = button->y;

    if ((ta.x <= x) && (x <= ta.x+ta.w))
      {
	if ((ta.y <= y) && (y <= ta.y + ta.h))
	  {
	    my_resizeMode = true;
	    my_resizeDelta.x = ta.x - x;
	    my_resizeDelta.y = ta.y - y;
	    Show();
	    SetCapture();
	    return true;
	  };
      }
    
    Show();
    return PG_Window::eventMouseButtonDown(button);
  }
  
  bool eventMouseButtonUp(const SDL_MouseButtonEvent *button)
  {
#if 0
    int x,y;
    
    x = button->x;
    y = button->y;
    
    x += my_resizeDelta.x;
    y += my_resizeDelta.y;
    
    if (x < 0) x=0;
    if (y < 0) y=0;
    if (x > (my_srfScreen->w - my_width - 1))
      x = (my_srfScreen->w - my_width - 1);
    if (y > (my_srfScreen->h - my_height - 1))
      y = (my_srfScreen->h - my_height -1);
#endif
    if (my_resizeMode)
      {
	my_resizeMode = false;
	ReleaseCapture();
	return true;
      }
    return PG_Window::eventMouseButtonUp(button);
  }
  
  bool eventMouseMotion (const SDL_MouseMotionEvent *motion)
  {
    if(!my_resizeMode) {
      return PG_Window::eventMouseMotion(motion);
    }
    int x,y;
    x = motion->x;
    y = motion->y;
    
    x += my_resizeDelta.x;
    y += my_resizeDelta.y;
    
    if (GetParent() != NULL)  // Not a top-level-widget
      {
	PG_Point pos = GetParent()->ScreenToClient(x,y);
	x = pos.x;
	y = pos.y;         // Should not be moved out of the parent:
	if (x >= GetParent()->Width() - 5) x = GetParent()->Width() - 5;
	if (y >= GetParent()->Height() - 5) y = GetParent()->Height() - 5;
      }
    else                      // A top-level-widget
      {
	if (x > (my_srfScreen->w - 1)) x = (my_srfScreen->w - 1);
	if (y > (my_srfScreen->h - 1)) y = (my_srfScreen->h - 1);
      }
    x -= this->x;
    y -= this->y;
    if (x < 16)    x=16;
    if (y < 16+25) y=16+25;

    SizeWidget(x, y);
    player->SizeWidget(w-4,h-29);
    return true;
  }
};

class MyApplication : public PG_Application, public PG_Navigator {
protected:
        bool isalt;
public:
        MyApplication() : PG_Application(), PG_Navigator() { isalt=false; };
protected:
	bool eventKeyDown(const SDL_KeyboardEvent* key);
};

bool MyApplication::eventKeyDown(const SDL_KeyboardEvent* key) {
	PG_Widget* dest = NULL;

	switch(key->keysym.sym) {
	        case SDLK_RALT:
	        case SDLK_LALT:
		  isalt=true;
		  break;
	        case SDLK_RETURN:
		  if (isalt)
		    {
		      isalt = false;
		      SDL_WM_ToggleFullScreen(GetScreen());
		    };
		  break;
		case SDLK_UP:
			dest = FindUp();
			break;
		case SDLK_DOWN:
			dest = FindDown();
			break;
		case SDLK_LEFT:
			dest = FindLeft();
			break;
		case SDLK_RIGHT:
			dest = FindRight();
			break;
		case SDLK_TAB:
			dest = GotoNext();
			if(dest == NULL) {
				GotoFirst();
			}
			return true;
		case SDLK_SPACE:
			Action(PG_ACT_OK);
			return true;
		default:
			dest = NULL;
			break;
	}

	if(dest != NULL) {
		Goto(dest);
		return true;
	}

	return false;
}

int main(int argc, char* argv[]) 
{
	if (argc != 2){
		printf("Use: %s archivo_mpeg\n", argv[0]);
		return -1;
	}

	// Uint32 flags = SDL_SWSURFACE;// | SDL_FULLSCREEN;
	// Uint32 flags = SDL_HWSURFACE;// | SDL_FULLSCREEN;
	Uint32 flags = 0;
        int bpp = 0;
	// PG_Application app;
	MyApplication app;
	/* flags |= SDL_RESIZABLE */ /* need hooks to make this useful */

	if(!app.LoadTheme("default")) {
                printf("Unable to load theme!\n");
                exit(-1);
        }

	if(!app.InitScreen(800, 600 , bpp, flags)){
                printf("Resolution not supported\n");
                exit(-1);
        }

	PG_Panel f(NULL, PG_Rect(700, 0, 100, 600));
	f.Show();

	int ypos = 10, height = 30, yskip = 5;

	PG_Button start(&f, 12, PG_Rect(10,ypos,80,height), "Start");
        start.Show();
	ypos += height + yskip;
	
	PG_Button stop(&f, 13, PG_Rect(10,ypos,80,height), "Stop");
        stop.Show();
	ypos += height + yskip;
	
	PG_Button rewind(&f, 14, PG_Rect(10,ypos,80,height), "Rewind");
        rewind.Show();
	ypos += height + yskip;
	
	PG_Button pause(&f, 15, PG_Rect(10,ypos,80,height), "Pause");
        pause.Show();
	ypos += height + yskip;
	
	PG_Button quit(&f, 16, PG_Rect(10,ypos,80,height), "Quit");
	quit.sigButtonClick.connect(slot(handle_exit), &app);
        quit.Show();
	ypos += height + yskip;

	PG_MPEGPlayerWindow *thePlayer;
#if 0
#define CMPEG PG_MPEGPlayerWindow
	// #define CMPEG PG_MoveableMPEGPLayerWindow
	// #define CMPEG PG_MyMPEGPlayerWindow
#if 0
	thePlayer = new CMPEG(argv[1], NULL, PG_Rect(100, 100, 640, 480));
#else
	PG_Panel fq(NULL, PG_Rect(0, 0, 700, 600-23));
	fq.Show();
	thePlayer = new CMPEG(argv[1], &fq, PG_Rect(100, 100, 640, 480));
#endif
#else
	PG_PlayerWindow vid(NULL, PG_Rect(100, 100, 640+4, 480+29), argv[1]);
	thePlayer = vid.player;
	vid.Show();
#endif
	thePlayer->SetTransparency(128);
	thePlayer->Show();

	PG_Slider vslider(NULL, -1, PG_Rect(10,600-22,700-22,20), PG_SB_HORIZONTAL);
	thePlayer->handle_position(&vslider);
	vslider.sigSlide.connect(slot(*thePlayer, &PG_MPEGPlayerWindow::handle_reposition));
	vslider.Show();

	PG_Window win(NULL, PG_Rect(200,150,200,150), "Overlap Window");
	PG_Label label(&win, PG_Rect(10,40,180,20), "Transparency");
	label.SetAlignment(PG_TA_CENTER);
	PG_Slider slider(&win, -1, PG_Rect(10,65,180,20), PG_SB_HORIZONTAL);
	slider.SetRange(0, 255);
	slider.SetPosition(128);		

	PG_CheckButton check(&win, -1, PG_Rect(10,90,180,20), "fast displaymode");

	start.sigButtonClick.connect(slot(*thePlayer, &PG_MPEGPlayerWindow::handle_start));
	stop.sigButtonClick.connect(slot(*thePlayer, &PG_MPEGPlayerWindow::handle_stop));
	rewind.sigButtonClick.connect(slot(*thePlayer, &PG_MPEGPlayerWindow::handle_rewind));
	pause.sigButtonClick.connect(slot(*thePlayer, &PG_MPEGPlayerWindow::handle_pause));
	slider.sigSlide.connect(slot(*thePlayer, &PG_MPEGPlayerWindow::handle_transparency));
	check.sigButtonClick.connect(slot(*thePlayer, &PG_MPEGPlayerWindow::handle_fastmode));
	
	win.SetTransparency(128);
	win.Show();

	app.Add(&start);
	app.Add(&stop);
	app.Add(&rewind);
	app.Add(&pause);
	app.Add(&quit);
	// app.Add(&slider);
	// app.Add(&vslider);
	app.GotoFirst();

	// thePlayer->SendToBack();
	app.Run();
        return EXIT_SUCCESS;
}
